/*
 xGL:ParticleSystem.h
 Created by mantis on 24/06/05.
 Copyright 2005 xonko interactive All rights reserved.
*/ 

#import <Cocoa/Cocoa.h>
#import "XGLGeometry.h"

typedef struct {
	vector position;
	vector rotation;
	vector scale;
	vector linearVelocity;
	vector angularVelocity;
	int lifeSpan;
	int life;	
} particle;

@interface XGLParticleSystem : XGLGeometry {
	
	vector emitter;
	vector gravity;
	vector wind;
	vector color;
	vector specialForce;
	
	float dragCoefficient;
	float maxVelocity;
	float minVelocity;
	int maxLifespan;
	int minLifeSpan;
	float maxScale;
	float minScale;
	
	NSMutableArray *plugins;
	float specialForceMultiplier;
	float specialForceThreshold;
	
	int numParticles;
	particle *particles;
	
}

//+(Class)class;
-(id)init;
-(void)drawGeometry;

- (int)numParticles;
- (void)setNumParticles:(int)newNumParticles;

- (float)dragCoefficient;
- (void)setDragCoefficient:(float)value;

- (float)maxVelocity;
- (void)setMaxVelocity:(float)value;

- (float)minVelocity;
- (void)setMinVelocity:(float)value;

- (int)maxLifespan;
- (void)setMaxLifespan:(int)value;

- (int)minLifeSpan;
- (void)setMinLifeSpan:(int)value;

- (float)maxScale;
- (void)setMaxScale:(float)value;

- (float)minScale;
- (void)setMinScale:(float)value;

//gravity KVC
- (float)gX;
- (void)setGX:(float)value;

- (float)gY;
- (void)setGY:(float)value;

- (float)gZ;
- (void)setGZ:(float)value;

//wind KVC
- (float)wX;
- (void)setWX:(float)value;

- (float)wY;
- (void)setWY:(float)value;

- (float)wZ;
- (void)setWZ:(float)value;

//emitter KVC
- (float)eX;
- (void)setEX:(float)value;

- (float)eY;
- (void)setEY:(float)value;

- (float)eZ;
- (void)setEZ:(float)value;

//color KVC

- (NSColor *)color;
- (void)setColor:(NSColor *)value;

//specialForce KVC

-(vector *)specialForce;
-(void)setSpecialForce:(vector)newForce;

-(float)specialForceMultiplier;
-(void)setSpecialForceMultiplier:(float)newMultiplier;

- (float)specialForceThreshold;
- (void)setSpecialForceThreshold:(float)value;



//motion modifiers
-(void)springForce:(int)aParticleNum;

//plugins KVC
- (NSArray *)plugins;
- (unsigned)countOfPlugins;
- (id)objectInPluginsAtIndex:(unsigned)theIndex;
- (void)getPlugins:(id *)objsPtr range:(NSRange)range;
- (void)insertObject:(id)obj inPluginsAtIndex:(unsigned)theIndex;
- (void)removeObjectFromPluginsAtIndex:(unsigned)theIndex;
- (void)replaceObjectInPluginsAtIndex:(unsigned)theIndex withObject:(id)obj;



/*
 
- (float)cR;
- (void)setCR:(float)value;

- (float)cG;
- (void)setCG:(float)value;

- (float)cB;
- (void)setCB:(float)value;

*/


@end
